#include <stdio.h>
#include <stdlib.h>
#include <math.h>

extern void sgelsd_(int *M, int *N, int *NRHS, float *A, int *LDA, float *B, int *LDB, float *S, float *RCOND, int *RANK, float *WORK, int *LWORK,
    int *IWORK, int *INFO);

static int c_sgelsd(int M, int N, int NRHS, float *A, int LDA, float *B, int LDB, float *S, float RCOND, int *RANK, float *WORK, int LWORK, int *IWORK)
{
    int INFO;
    sgelsd_(&M,&N,&NRHS,A,&LDA,B,&LDB,S,&RCOND,RANK,WORK,&LWORK,IWORK,&INFO);

    return INFO;
}

extern void sgels_(char *TRANS, int *M, int *N, int *NRHS, float *A, int *LDA, float *B, int *LDB, float *WORK, int *LWORK, int *INFO);

static int c_sgels(char *TRANS, int M, int N, int NRHS, float *A, int LDA, float *B, int LDB, float *WORK, int LWORK)
{
    int INFO;
    sgels_(TRANS,&M,&N,&NRHS,A,&LDA,B,&LDB,WORK,&LWORK,&INFO);

    return INFO;
}

extern void dgels_(char *TRANS, int *M, int *N, int *NRHS, double *A, int *LDA, double *B, int *LDB, double *WORK, int *LWORK, int *INFO);

static int c_dgels(char *TRANS, int M, int N, int NRHS, double *A, int LDA, double *B, int LDB, double *WORK, int LWORK)
{
    int INFO;
    dgels_(TRANS,&M,&N,&NRHS,A,&LDA,B,&LDB,WORK,&LWORK,&INFO);

    return INFO;
}

int main(void)
{
    int M, N;
    int INFO;
    double minlwork = 0;
    double * WORK = NULL;
    double *A = NULL, *B = NULL;

    {
        FILE *deltas = fopen("delta.dmp", "rb");
        if (!deltas) return 1;

        fread(&M, sizeof(M), 1, deltas);
        fread(&N, sizeof(N), 1, deltas);
        
        A = malloc(sizeof(double)*M*N);
        B = malloc(sizeof(double)*M);

        if (!A || !B) return 1;

        for (int i = 0; i < M; i++)
        {
            fread(&B[i], sizeof(double), 1, deltas);
        }

        for (int i = 0; i < M*N; i++)
        {
            fread(&A[i], sizeof(double), 1, deltas);
        }

        fclose(deltas);
    }

    INFO =
    c_dgels("N",   //TRANS
           M,       //M
           32,      //N
           1,       //NRHS
           A,       //A
           M,       //LDA
           B,       //B
           M,       //LDB
           &minlwork,//WORK
           -1       //LWORK
           );

    if (INFO != 0)
    {
        fprintf(stderr, "INFO=%d\n",INFO);
        return 1;
    }

    WORK = calloc(minlwork, sizeof(double));

    INFO =
    c_dgels(
           "N",     //TRANS
           M,       //M
           32,      //N
           1,       //NRHS
           A,       //A
           M,       //LDA
           B,       //B
           M,       //LDB
           WORK,    //WORK
           minlwork //LWORK
           );
    free(WORK); WORK=NULL;

    for (int j = 0; j <= 8; j++)
    {
    for (int i = 0; i < 32; i++)
    {
        //printf("%d,\n", (int)B[i]);
        printf("%d,", (int)round(B[i]*(1<<j)));
        //printf("%lf,", B[i]);
    }
    printf("\n");
    }

    return 0;
}
